@@ -1,7 +1,7 @@
-from __future__ import annotations
+# from __future__ import annotations
 
 from dataclasses import field
-from typing import Any, Optional
+from typing import Any, Dict, List, Optional, Set
 
 # Use pydantic for type checking. It is slower, but not much of an issue for parsing.
 from pydantic.dataclasses import dataclass
@@ -13,7 +13,7 @@ from string import ascii_lowercase, ascii_uppercase, digits, punctuation
 
 #region Helper functions
 
-def _to_json_compatible_object(obj: dataclass) -> dict[str,Any]:
+def _to_json_compatible_object(obj: dataclass) -> Dict[str,Any]:
   """
   Converts arbitrary objects to json suitable objects. Tries to convert types intelligently.
 
@@ -21,7 +21,7 @@ def _to_json_compatible_object(obj: dataclass) -> dict[str,Any]:
       obj (Any): Object to convert to json.
 
   Returns:
-      dict[str,Any]: Object suitable for use with ``json.dumps``.
+      Dict[str,Any]: Object suitable for use with ``json.dumps``.
   """
   data = {}
   
@@ -48,12 +48,12 @@ def _to_json_compatible_object(obj: dataclass) -> dict[str,Any]:
   return data
 
 
-def _uses_alphabet_charset(rules: list[PCPRule]) -> bool:
+def _uses_alphabet_charset(rules: List['PCPRule']) -> bool:
   """
   Checks if the given rules use the alphabet charset.
 
   Args:
-      rules (list[PCPRule]): List of rules to check.
+      rules (List[PCPRule]): List of rules to check.
 
   Returns:
       bool: Whether the rules use the alphabet charset.
@@ -71,14 +71,14 @@ def _uses_alphabet_charset(rules: list[PCPRule]) -> bool:
 
 #region PCP
 
-DEFAULT_CHARSETS: dict[str,str] = {
+DEFAULT_CHARSETS: Dict[str,str] = {
   "lower": ascii_lowercase,
   "upper": ascii_uppercase,
   "digits": digits,
   "symbols": punctuation  
 }
 
-ALPHABET_CHARSETS: dict[str,str] = {
+ALPHABET_CHARSETS: Dict[str,str] = {
   "alphabet": ascii_lowercase + ascii_uppercase,
   "digits": digits,
   "symbols": punctuation  
@@ -93,8 +93,8 @@ class PCP():
   
   ``charsets`` is the list of supported character sets. By default, includes upper, lower, digits, and symbols. Character sets can be redefined, but character sets must not overlap. An empty character set cannot be a part of a PCP.
   """
-  rules: list[PCPRule]
-  charsets: dict[str,str] = field(default_factory=DEFAULT_CHARSETS.copy)
+  rules: List['PCPRule']
+  charsets: Dict[str,str] = field(default_factory=DEFAULT_CHARSETS.copy)
   
   def dumps(self) -> str:
     """Create a json representation of this object.
@@ -111,7 +111,7 @@ class PCP():
         return json.dumps({'rules': [_to_json_compatible_object(rule) for rule in self.rules]})
 
     # Get a diff between the defined charset and the default charset. Only dump this diff.
-    charsets_diff: dict[str,str] = {}
+    charsets_diff: Dict[str,str] = {}
     default_charset = ALPHABET_CHARSETS if 'alphabet' in self.charsets else DEFAULT_CHARSETS
         
     for name, charset in self.charsets.items():
@@ -126,7 +126,7 @@ class PCP():
     
     
   @staticmethod
-  def loads(s: str) -> PCP:
+  def loads(s: str) -> 'PCP':
     """
     Load a PCP object from a json string.
 
@@ -139,14 +139,14 @@ class PCP():
     data = json.loads(s)
     
     # Parse the rules
-    rules: list[PCPRule]
+    rules: List[PCPRule]
     if 'rules' not in data or data['rules'] is None:
       rules = [PCPRule._from_json(data)]
     else:
       rules = [PCPRule._from_json(rule) for rule in data['rules']]
     
     # Parse the charsets
-    charsets: dict[str, str] = ALPHABET_CHARSETS.copy() if _uses_alphabet_charset(rules) else DEFAULT_CHARSETS.copy()
+    charsets: Dict[str, str] = ALPHABET_CHARSETS.copy() if _uses_alphabet_charset(rules) else DEFAULT_CHARSETS.copy()
     if 'charsets' in data and data['charsets'] is not None:
       for key, value in data['charsets'].items():
         if value is None or len(value) == 0:
@@ -201,20 +201,20 @@ class PCPRule():
   min_length: int = 1
   max_length: Optional[int] = None
   max_consecutive: Optional[int] = None
-  prohibited_substrings: Optional[set[str]] = None
+  prohibited_substrings: Optional[Set[str]] = None
   
-  require: Optional[set[str]] = None
-  require_subset: Optional[PCPSubsetRequirement] = None
+  require: Optional[Set[str]] = None
+  require_subset: Optional['PCPSubsetRequirement'] = None
   
-  charset_requirements: Optional[dict[str,PCPCharsetRequirement]] = None
+  charset_requirements: Optional[Dict[str,'PCPCharsetRequirement']] = None
   
   @staticmethod
-  def _from_json(data: dict[str,Any]) -> PCPRule:
+  def _from_json(data: Dict[str,Any]) -> 'PCPRule':
     """
     Load a PCPRule from a json-compatible object.
 
     Args:
-        data (dict[str,Any]): Object to parse.
+        data (Dict[str,Any]): Object to parse.
 
     Returns:
         PCPRule: Parsed PCPRule.
@@ -232,13 +232,13 @@ class PCPRule():
       }
     return PCPRule(**data)
     
-  def _validate(self, argname: str, charsets: set[str]) -> None:
+  def _validate(self, argname: str, charsets: Set[str]) -> None:
     """
     Validates that the rule is self consistent.
 
     Args:
         argname (str): Name of this object within the policy.
-        charsets (set[str]): Charsets defined by the policy.
+        charsets (Set[str]): Charsets defined by the policy.
 
     Raises:
         ValueError: Describes how the policy is malformed.
@@ -293,14 +293,14 @@ class PCPSubsetRequirement():
   ``options`` is the set of character sets from which characters must be selected. If unset, all charsets will be used as valid options.
   """
   count: int
-  options: Optional[set[str]] = None
+  options: Optional[Set[str]] = None
   
   @staticmethod
-  def _from_json(data: dict[str,Any]) -> PCPSubsetRequirement:
+  def _from_json(data: Dict[str,Any]) -> 'PCPSubsetRequirement':
     """Load a PCPSubsetRequirement from a json-compatible object.
 
     Args:
-        data (dict[str,Any]): Object to parse.
+        data (Dict[str,Any]): Object to parse.
 
     Returns:
         PCPSubsetRequirement: Parsed PCPSubsetRequirement.
@@ -309,12 +309,12 @@ class PCPSubsetRequirement():
       data['options'] = set(data['options'])
     return PCPSubsetRequirement(**data)
     
-  def _validate(self, argname: str, charsets: set[str]) -> None:
+  def _validate(self, argname: str, charsets: Set[str]) -> None:
     """Validates that the subset requirement is self consistent.
 
     Args:
         argname (str): Name of this object within the policy.
-        charsets (set[str]): Charsets defined by the policy.
+        charsets (Set[str]): Charsets defined by the policy.
 
     Raises:
         ValueError: Describes how the policy is malformed.
@@ -351,16 +351,16 @@ class PCPCharsetRequirement():
   min_required: Optional[int] = None
   max_allowed: Optional[int] = None
   max_consecutive: Optional[int] = None
-  required_locations:  Optional[set[int]] = None
-  prohibited_locations: Optional[set[int]] = None
+  required_locations:  Optional[Set[int]] = None
+  prohibited_locations: Optional[Set[int]] = None
 
   @staticmethod
-  def _from_json(data: dict[str,Any]) -> PCPCharsetRequirement:
+  def _from_json(data: Dict[str,Any]) -> 'PCPCharsetRequirement':
     """
     Load a PCPCharsetRequirement from a json-compatible object.
 
     Args:
-        data (dict[str,Any]): Object to parse.
+        data (Dict[str,Any]): Object to parse.
 
     Returns:
         PCPCharsetRequirement: Parsed PCPCharsetRequirement.
